; example for the controller-board (www.frydaysoft.de)
;
; Simple source-code for multiplexed 8 x 7segment-LED's.
; Max. visible value is FFFFFFFF.
; 7seg-modul connected with SL1 (uC-board).
; Timer0 diplays 32bit-value on connected modul.
;
; Led D4 flashes
; Key S4: Single-increment
; Key S3: auto-increment
; Key S2: quick auto-increment


$include (mod51)	; Register-defs

;**** Bit-RAM 20..2F ****
 BSEG AT 02Fh		; use only last byte of BSEG-block     
  key_pressed: DBIT 1	

;**** Byte-RAM from 8h upto 2Dh ****
 DSEG AT 8h		; bank1..3 8..1Fh
  bcd_buf:  DS  5h	; 8 BCD-nibbles for 7Seg-module
			; and one internal member-byte
 DSEG AT 30h		; 30..7Fh
  STACK_RAM:   DS 50h	; **** reserved STACK-area  ****


;**** Reg-zuweisung ****
Key4   SET   P1.0
Key3   SET   P1.1
Key2   SET   P1.2
Key1   SET   P1.3
Led4   SET   P1.4
Led3   SET   P1.5
Led2   SET   P1.6
Led1   SET   P1.7

tmppointer	equ R0
;R1..R7 not used

waitconst 	equ  80h       	;const for wait-looping
Stack_First 	equ  30h     	;set stackspointer to reserved RAM

;defines Timer0: (defined multiplexing-speed)
TH0Value equ 0F4h                ;16BIT-value timer0
TL0Value equ 000h                

;***Pins of HCT138 (IC6)***********************************************************************
HCT138_Y1	equ  10010000b


;##################### MAIN Program #######################################

cseg at 0
	;USING 3                 
	;USING 2                 
	;USING 1                 
	USING 0     	;Bank 0

org 0000h
jmp main_start		;jump to program

org 0003h               ;extInt0
reti

org 000bh               ;timer0
jmp T0_INT

org 0013h               ;extInt1
reti

org 001Bh               ;timer1
reti

org 0023h               ;RS232
reti

;**************************************************************************

main_start:
	mov SP,  #Stack_First
	mov DPTR,#0
	mov acc, #0
	mov b,	 #0
	mov PSW, #0
	mov PCON,#0
	mov SCON,#0
	mov TCON,#0
	mov TMOD,#0
	mov IE,  #0
	mov IP,  #0

	clr EA
	lcall init_t0           ;init timer0
	mov bcd_buf,#00h
	mov bcd_buf+1,#00h
	mov bcd_buf+2,#00h
	mov bcd_buf+3,#00h
	mov bcd_buf+4,#0

        mov TL0,#TL0Value       ;reinit T0-Counters
	mov TH0,#TH0Value
        setb TR0
	setb EA			;mainswitch INTR on
	mov P1,#0ffh		
	clr key_pressed

;################## Main - L O O P ########################

MainLoop:
	xrl P1,#10h			;flash LED4
	jnb KEY2,MainLoop4
	jnb KEY3,MainLoop3
	jb KEY4,MainLoop2
	jb key_pressed,MainLoop
	inc bcd_buf			;single-inc.
	mov a,bcd_buf
	jnz MainLoop1
	inc bcd_buf+1
	mov a,bcd_buf+1
	jnz MainLoop1
	inc bcd_buf+2
	mov a,bcd_buf+2
	jnz MainLoop1
	inc bcd_buf+3
MainLoop1:
	setb key_pressed
	jmp MainLoop	
MainLoop2:
	clr key_pressed
	call wait
	jmp MainLoop
MainLoop3:
	call shortwait
MainLoop4:
	inc bcd_buf			;auto-inc.
	mov a,bcd_buf
	jnz MainLoop
	inc bcd_buf+1
	mov a,bcd_buf+1
	jnz MainLoop
	inc bcd_buf+2
	mov a,bcd_buf+2
	jnz MainLoop
	inc bcd_buf+3
	jmp MainLoop

;****************************************************************

init_T0:
	anl TMOD,#11110000b    ;16 bit Timer0 (gate C/T MODE)
	orl TMOD,#00001001b    ;Gate0 (01-16-bit 10-8bit reload)
	mov TL0,#TL0Value
	mov TH0,#TH0Value 
	clr TF0
	setb TR0
	setb ET0
	setb PT0		;higher priority
	ret

;################ Interrupts ###################

; internal counter (bcd_buf+4)
; D0: high (D0=1) or low-nibble (D0=0)
; D1,2: Index of 4Byte-array 
; D3..7: unused

T0_INT:

	push acc
        push PSW			;store register
        push b
	mov a,tmppointer
        push acc

	mov a,bcd_buf+4			;internal counter
	rr a				;shift right
	anl a,#3h			;get masked index (0..3)
	add a,#bcd_buf			;get addr. of active 7Seg-byte
	mov tmppointer,a
	mov b,@tmppointer		;get data of active byte
	mov a,bcd_buf+4
	jb acc.0,T0_INT_HighNibbel	
	anl b,#0Fh			;only LOW-nibble
	jmp T0_INT_7Seg
T0_INT_HighNibbel:
	anl b,#0F0h			;only HIGH-nibble
	mov a,b
	swap a				;copy into LOW-part
	mov b,a
T0_INT_7Seg:
	mov a,bcd_buf+4
	anl a,#7h			;Position of 7Seg (0..7)
	
	call put_7seg			;accu 0..7 for 7Seg-pos and b=BCD-code
T0_Ret:
	inc bcd_buf+4			;update internal counter
        pop acc				
	mov tmppointer,a		;restore register
        pop b
        pop PSW
        pop acc

	mov TL0,#TL0Value
	mov TH0,#TH0Value 
	;setb TR0
	reti

;****************************************************************

$include (waitloop.inc)
$include (io_port.inc)
$include (7seg.inc)

end
